package com.example;

import akka.actor.typed.ActorRef;
import akka.actor.typed.Behavior;
import akka.actor.typed.javadsl.AbstractBehavior;
import akka.actor.typed.javadsl.ActorContext;
import akka.actor.typed.javadsl.Behaviors;
import akka.actor.typed.javadsl.Receive;
import akka.actor.typed.javadsl.TimerScheduler;

import java.util.ArrayList;
import java.util.List;

public class DeliveryCar extends AbstractBehavior<DeliveryCar.Message> {

    private final ArrayList<Paket> pakete = new ArrayList<>();
    private final ArrayList<ActorRef<Customer.Message>> route;
    private final int maxCapacity = 3;
    private int globalIndex = 0;

    public interface Message {}

    public record LoadMessage(List<Paket> pakete) implements Message {}

    public record PickupResponse(Paket paket) implements Message {}

    public record StartRoute() implements Message {}
    
    private DeliveryCar(ActorContext<Message> context, ArrayList<ActorRef<Customer.Message>> route) {
        super(context);
        this.route = route;
    }

    public static Behavior<Message> create(List<ActorRef<Customer.Message>> route) {
        return Behaviors.setup(context -> new DeliveryCar(context, new ArrayList<>(route)));
    }

    @Override
    public Receive<Message> createReceive() {
        return newReceiveBuilder()
                .onMessage(PickupResponse.class, this::onPickupResponse)
                .onMessage(LoadMessage.class, this::onLoadMessage)
                .onMessage(PickupResponse.class, this::onPickupResponse)
                .onMessage(StartRoute.class, msg->{
                    onStartRoute();return this;})
                .build();
    }

    private Behavior<Message> onLoadMessage(LoadMessage msg) {
        pakete.addAll(msg.pakete);
        checkNextStop();
        return this;
    }

    private Behavior<Message> onPickupResponse(PickupResponse rsp) {
        if(rsp.paket.inhalt == null){
            globalIndex++;
            checkNextStop();
        }else {
            if(pakete.size()==3){
                checkNextStop();
            }
            pakete.add(rsp.paket);
            getContext().getLog().info("Aktuelle Anzahl der Pakete im Truck: {}", pakete.size());
            globalIndex++;
            checkNextStop();
        }
        return this;
    }
//Die rekursive Methode,die über alle Addressen iteriert
    private void checkNextStop() {
        //Wenn Wagen voll ist, dann wird erstmal geguckt ob man ein Paket liefern kann
        if (pakete.size() >= maxCapacity) {
            deliverPackages();
            checkNextStop();
        }
        //Wenn fertig iteriert so geht der Wagen an den Verteilerzentrum
        if(globalIndex==4){
            DistributionCenter.ArriveMessage antwort = new DistributionCenter.ArriveMessage(pakete,getContext().getSelf());
            //ActorRef<DistributionCenter.Message> antwort1;
           // antwort1.tell(antwort);

          //Fehlende Antwort an den Verteilerzentrum


            //

        }
        else {
            //Guckt ob Paket für Kunden da ist und fragt an ob etwas mitgenommen werden muss
            deliverPackages();
            sendPickupMessage();
        }
    }
    public Behavior<Message> onStartRoute(){
        /*if(globalIndex==0){
            checkNextStop();
            return this;

        }else{
            if(globalIndex==4) {
                globalIndex = 0;
                onStartRoute();
            }
        }
          return this;*/
        globalIndex = 0;
        checkNextStop();
        return this;
    }

//Sendet PickUp Nachricht an den jetztigen Kunden wo der gerade ist.
    private void sendPickupMessage() {
        ActorRef<Customer.Message> nextCustomer = route.get(globalIndex);
        nextCustomer.tell(new Customer.PickUpMessage(getContext().getSelf(), "Wagen kann Paket aufnehmen"));
        globalIndex++;
    }


    //Prüft ob Paket im Wagen für den Empfänger ist oder nicht, falls ja wird dieser direkt entfernt
    private void deliverPackages() {
        ActorRef<Customer.Message> nextCustomer = route.get(globalIndex);
        //Iteriert über alle Pakete
        for (Paket paket : pakete) {
            //Falls Empfänger vorhanden so wird dieser aus dem Wagen genommen und geht dann weiter
            if(pakete.get(globalIndex).empfaenger == nextCustomer ) {
                nextCustomer.tell(new Customer.DeliveryMessage(paket));
                pakete.remove(globalIndex);
               // globalIndex++;
            }
        }

    }

    private void removeMessage(Paket paket){
        pakete.remove(paket);
    }


}